/*
 * Copyright (c) 1994 Hellmuth Michaelis
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * @(#)upsstat.c, 1.00, Last Edit-Date: [Tue Aug  9 17:16:35 1994]
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	get Uninteruptable Power Supply status
 *	--------------------------------------
 *
 *	Hellmuth Michaelis, hm@hcshh.hcs.de
 *
 *	last edit-date: [Fri Aug  5 15:23:52 1994]
 *
 *	-hm	stripped down daemon
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

#include "upsd.h"
#include "upsbits.h"

void	usage(void);

extern char *optarg;

/*---------------------------------------------------------------------------*
 *	main
 *---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
	unsigned char upsbyte;
	int	filed;
	int	c;
	int	verbose = 0;
	int	raw = 0;

	char	*port = UPSDEVICE;
	
	/* parse the command line */

	while ((c = getopt(argc, argv, "vr")) != EOF)
	{
		switch (c)
		{
			case 'v':		/* verbose messages */
				verbose = 1;
				break;
				
			case 'r':		/* "raw" device */
				port = UPSRDEVICE;			
				raw = 1;
				break;

			default:
				usage();	/* does not return */
				break;			
		}
	}

	/* open the device to get status info from */
	
	if((filed = open(port, O_RDONLY)) == -1)
	{
		if(verbose)
			fprintf(stderr, "upstat: cannot open device %s: %s\n", port, sys_errlist[errno]);
		exit(2);		/* 2 = device open error */
	}	
	
	/* read the current status byte from ups driver */

	if((read(filed, &upsbyte, 1)) != 1 )
	{
		if(verbose)
			fprintf(stderr, "upsstat: cannot read device %s: %s\n", port, sys_errlist[errno]);
		exit(3);		/* 3 = read error */
	}

	close(filed);

	if(raw)
	{
		fprintf(stderr, "upsstat: raw status byte = 0x%02x\n", upsbyte);
		exit(5);		/* 5 = raw status */
	}
	else
	{
		if(verbose)
		{
			fprintf(stderr, "upsstat: generic status byte = 0x%02x\n", upsbyte);
			if(upsbyte & UPS_POBATEMPT)
				fprintf(stderr, "upsstat: battery was empty at power-on time\n");
			else
				fprintf(stderr, "upsstat: battery was full at power-on time\n");

			if(upsbyte & UPS_BATTEMPTY)
				fprintf(stderr, "upsstat: battery is empty\n");
			else
				fprintf(stderr, "upsstat: battery is full\n");
		}

		if(upsbyte & UPS_POWERFAIL)
		{
			if(verbose)
				fprintf(stderr, "upsstat: powerfail state, running from battery!\n");
			exit(1);		/* 1 = powerfail, battery power */
		}
		else
		{
			if(verbose)
				fprintf(stderr, "upsstat: AC power ok, running from wall power.\n");
			exit(0);		/* 0 = OK, wall power */
		}
	}
}

/*---------------------------------------------------------------------------*
 *	usage
 *---------------------------------------------------------------------------*/
void usage(void)
{
	fprintf(stderr, "\nupsstat - return UPS status 1.00\n");
	fprintf(stderr, "usage: upsstat [-v] [-r]\n");
	fprintf(stderr, "       -v = interactive use, issue status messages\n");
	fprintf(stderr, "       -r = use raw device, supply hw dependent status byte\n");
	exit(4);	/* 4 = usage error */
}	

/* EOF */
